@echo off
REM Win2000 SP4 first-time installation procedure consists of multiple steps:
REM  Uninstall TSClient (due to problems updating it after SP4 is installed)
REM  Install locally
REM  Install W2KAdminTools SP4 (to skip this specify NOADMIN in %1) - actually at least with SP2,
REM   ADMINPAK.MSI does not install until after a reboot, so we copy it to %SystemRoot%\system32
REM   and set up a RunOnce Key to automatically install the tools after the reboot
REM  Update any i386 folders: SP4\update\Update.exe -s:C:\Software
REM   and set the SourcePath & ServicePackSourcePath registry values to the path found
REM  Update the Recovery Console (if it is already installed) from the above folders
REM   (in order for this to be successful, C: cannot be mirrored, so break the mirror before running this batch file)
REM When upgrading domain controllers from NT4 to Win2000, SP1 or later needs to be installed
REM prior to Active Directory (DCPROMO) due to some bugs in DCPROMO when it comes to
REM domain names and such that are fixed in SP1.  Therefore, Win2000 SP4 installation
REM is now modularized into this batch file.  This batch file can be executed either
REM via RunMe or manually, in which case, this batch file needs to be executed from
REM \TOOLS\AM or from [AM Tech/Patch CD]\AM

REM You can specify an optional second parameter of FORCE (anything nonblank actually)
REM to force this batch file to install SP4 even if it detects some problem such as
REM a previous setup after which a reboot has not yet been performed.

set COPYCMD=/Y

if "%RMSOURCE%"=="" set RMSOURCE=%~dp0..

if not exist %RMSOURCE%\patches\nt\2000\W2KSP4_EN.EXE if not exist %RMSOURCE%\Private\Software\Microsoft\Win2000\SP4\i386\update\update.exe goto skipsp4

REM Uninstall TSClient
REM Old line - doesn't actually work due to lack of source files and it's the wrong TSClient anyway:
REM start /wait "" msiexec /x{B6CAA8E1-4F33-4208-B25E-0376200202D0} /qb-
if exist "%ProgramFiles%\Terminal Services Client\setup\SETUP.EXE" start /wait "" "%ProgramFiles%\Terminal Services Client\setup\SETUP.EXE" /u /qnt

set SP4SOURCE=%RMSOURCE%\Private\Software\Microsoft\Win2000\SP4\i386
if exist %SP4SOURCE%\update\update.exe goto nosp4extract
md "%temp%\patchtmp" 2>nul
start /wait "" %RMSOURCE%\patches\nt\2000\W2KSP4_EN.EXE /q /x:"%temp%\patchtmp\SP4"
set SP4SOURCE=%temp%\patchtmp\SP4\i386
:nosp4extract

IF /I /%1==//skip goto skip
IF /I -%1==--skip goto skip

start /wait "" "%SP4SOURCE%\update\update.exe" -u -o -z
REM If SP4 detects that the Pending File Rename Queue is not empty it will not run
REM (in 'non-unattended' mode it displays an error message)
REM However, if you rerun it again, it will then go ahead and run
REM Since the runme takes care to install things in the proper sequence
REM it's OK to force the install of SP4 via the runme but when this batch file
REM is executed manually, we return the same error message as the service pack
REM would in 'non-unattended' mode.
REM New version of Update.exe used in SP4 only sets ERRORLEVEL to 0 if no reboot is required!
REM This check will only trigger on error, not on 'SUCCESS, REBOOT REQUIRED'
IF ERRORLEVEL 1603 IF NOT ERRORLEVEL 3010 if !%2==! goto error
IF ERRORLEVEL 1603 IF NOT ERRORLEVEL 3010 if NOT !%2==! start /wait "" "%SP4SOURCE%\update\update.exe" -u -o -z
REM If the user cancells the SP4 install, it will re-try again but this time it will
REM most likely detect an 'unfinished prior install' again and quitely abort
REM In this case, we don't want to proceed with the remaining SP4 items and we return error msg
IF ERRORLEVEL 1603 IF NOT ERRORLEVEL 3010 if !%2==! goto error
:skip

If /I !%1==!NOADMIN goto noadmin
REM start /wait "" msiexec /i %SP4SOURCE%\ADMINPAK.MSI /qb-
REM Actually at least with SP2,
REM  ADMINPAK.MSI does not install until after a reboot, so we copy it to %SystemRoot%\system32
REM  and set up a RunOnce Key to automatically install the tools after the reboot
set SystemDir=%windir%\system32
set REG_SystemDir=%SystemDir:\=\\%
copy %SP4SOURCE%\ADMINPAK.MSI %SystemDir%
echo Windows Registry Editor Version 5.00>%temp%\ADMINPAK.reg
echo.>>%temp%\ADMINPAK.reg
echo [HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\RunOnce]>>%temp%\ADMINPAK.reg
echo "!W2KAdminTools"="msiexec /i %REG_SystemDir%\\ADMINPAK.MSI /qb">>%temp%\ADMINPAK.reg
echo.>>%temp%\ADMINPAK.reg
start /wait "" regedit /s %temp%\ADMINPAK.reg
del %temp%\ADMINPAK.reg
:noadmin

REM Get local fixed drives and see if an i386 distribution point exists on each one
REM The last one found (in \i386 or \Software\i386) is the one that's updated
set SP4_INTEG=
FOR /F "tokens=4" %%I IN ('%~0\..\wbemdump /Q root\cimv2 WQL "SELECT DeviceID FROM Win32_LogicalDisk WHERE DriveType = '3'" ^|find "*"') DO call :sp4integ %%I
goto end_sp4integ
:sp4integ
REM WBEMDUMP returns values quoted, so need to strip quotes to get C:, D:, E:, etc.
set SWDRIVE=%1
set SWDRIVE=%SWDRIVE:"=%
if exist %SWDRIVE%\i386\winnt32.exe set SP4_INTEG=%SWDRIVE%
if exist %SWDRIVE%\Software\i386\winnt32.exe set SP4_INTEG=%SWDRIVE%\Software
set SWDRIVE=
goto :EOF
:end_sp4integ
IF DEFINED SP4_INTEG start /wait "" "%SP4SOURCE%\update\update.exe" -u -s:%SP4_INTEG%\

IF NOT DEFINED SP4_INTEG goto noRecoveryConsole

REM Set the SourcePath and ServicePackSourcePatch to the found integrated source:
del %temp%\setupsrc.reg 2>nul
set REG_SP4_INTEG=%SP4_INTEG:\=\\%\\
echo REGEDIT4>%temp%\setupsrc.reg
echo.>>%temp%\setupsrc.reg
echo [HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Setup]>>%temp%\setupsrc.reg
echo "ServicePackSourcePath"="%REG_SP4_INTEG%">>%temp%\setupsrc.reg
echo "SourcePath"="%REG_SP4_INTEG%">>%temp%\setupsrc.reg
echo.>>%temp%\setupsrc.reg
start /wait "" regedit /s %temp%\setupsrc.reg
del %temp%\setupsrc.reg 2>nul
set REG_SP4_INTEG=

if exist %SystemDrive%\cmdcons\ntdetect.com start /wait "" %SP4_INTEG%\i386\winnt32.exe /cmdcons /unattend

:noRecoveryConsole
set SP4_INTEG=

if NOT %SP4SOURCE%==%temp%\patchtmp\SP4\i386 goto skipsp4
rmdir /q /s %temp%\patchtmp

goto skipsp4

:error
set SP4SOURCE=
set SP4_INTEG=
echo.
echo *******************************************************************************
echo *                                                                             *
echo *                          Service Pack Setup Error                           *
echo *                                                                             *
echo *******************************************************************************
echo *                                                                             *
echo *  The system must be restarted before installing the Service Pack, to allow  *
echo *   some prior file update operations to complete.  (These operations were    *
echo *     previously scheduled by some other install or uninstall operation.)     *
echo *                                                                             *
echo *******************************************************************************
echo.
pause
EXIT /B 1

:skipsp4
set SP4SOURCE=
set SP4_INTEG=
EXIT /B 0

